@@ -85,7 +85,7 @@ class AuthenticationClient(nio.AsyncClient):
 class Client(AuthenticationClient):
     MIN_RETRY_TIME = 3
     MAX_RETRY_TIME = 300
-    REQUEST_TIMEOUT = 60
+    REQUEST_TIMEOUT = 30000
     CONSIDER_SUCCESSFUL = 10
 
     def __init__(self, server: str, handle: str, session: "Session"):
@@ -302,14 +302,18 @@ class Client(AuthenticationClient):
                 s, room.room_id, msg_id, sender, with_event_ids=True
             )
 
-        if event.event_id in reactions.values():
-            if reactions[event.event_id] != emoji:
+        event_ids = list(reactions.values())
+        emojis = list(reactions.keys())
+        if event.event_id in event_ids:
+            idx = event_ids.index(event.event_id)
+            stored_emoji = emojis[idx]
+            if stored_emoji != emoji:
                 warnings.warn(
                     f"Received an emoji reaction update for the event ID '{event.event_id}': "
-                    f"'{reactions[event.event_id]}' → '{emoji}'. This is illegal in Matrix, ignoring."
+                    f"'{stored_emoji}' → '{emoji}'. This is illegal in Matrix, ignoring."
                 )
         else:
-            if emoji in reactions:
+            if emoji in emojis:
                 warnings.warn(
                     f"We already had an event ID for the emoji reaction '{emoji}':"
                     f" '{event.event_id}' → {reactions[emoji]}."
@@ -324,10 +328,10 @@ class Client(AuthenticationClient):
                     f"The emoji '{emoji}' was already present with a different event ID"
                 )
             else:
-                reactions[emoji] = event.event_id
+                emojis.append(emoji)
 
         participant = await self.get_participant(room, event)
-        participant.react(msg_id, list(reactions), **kw)
+        participant.react(msg_id, emojis, **kw)
 
     @catch_all
     async def on_redact(self, room: nio.MatrixRoom, event: nio.RedactionEvent):
